/* USER CODE BEGIN Header */
/**
 ******************************************************************************
 * @file           : main.c
 * @brief          : Main program body
 ******************************************************************************
 * @attention
 *
 * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
 * All rights reserved.</center></h2>
 *
 * This software component is licensed by ST under BSD 3-Clause license,
 * the "License"; You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at:
 *                        opensource.org/licenses/BSD-3-Clause
 *
 ******************************************************************************
 */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32cubeide_init.h"
#include "stm32f3xx_nucleo.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "Arduino.h"
#include "board.h"
#include "SPI.h"

#include "Wire.h"
#define SLAVE_ADDR_8574_A 0x38+6
#define SLAVE_ADDR_8574_B 0x38+7
    //////////////////////////////////////////
    char readPort8574(char addr, char * ptr_value)
    /*addr, l'adresse du PCF8574
    ptr_value, pointeur pour renvoyer la valeur lue sur le port
    retourne -1 si échec 0 sinon*/
    {
        Wire.requestFrom((byte)addr, (byte)1);// demande la lecture d'1 octet depuis l'adresse du pérpiphérique
        if (Wire.available()==1) //si l'octet est disponible
        {
            (* ptr_value) = Wire.read(); // lire l'octet
            return 0;
        }
        else
        {
            (* ptr_value) =0; //valeur par défaut si le composant n'a pas acquité
            return -1;
        }
    }
    //////////////////////////////////////////
    char writePort8574(char addr, char value)
    /*addr, l'adresse du PCF8574
    value, la valeur à écrire sur le port
    retourne -1 si échec 0 sinon */
    {
        Wire.beginTransmission(addr);//démarre la transmission avec l'adresse du pérpiphérique
        Wire.write((byte)value);     //envoie la donnée
        if (Wire.endTransmission()==0)      //stoppe la transmission
            return 0;
        else
            return -1;
    }
/* USER CODE END Includes */

/* USER CODE BEGIN 0 */

const uint32_t Tech = 1000 ;	// Tech en microsecondes !
const uint8_t resConv = 10 ;	// resolution des convertisseurs
const uint16_t offsetNum = (1 << (resConv-1) );	// calcul de l'offset à soustraire des échantillons d'entrée
const uint16_t maxConv = (1 << resConv) - 1 ;

const uint16_t ledPin = LED_BUILTIN;	// les broches utilisées
const uint16_t boutonPin = 9;
const uint16_t visuPin = 2;
const uint16_t ADCPin = A4;
const uint16_t DACPin = A2;

/* USER CODE END 0 */

/**
 * @brief  The application entry point.
 * @retval int
 */
int main(void)
{
	/* USER CODE BEGIN 1 */
	/* USER CODE END 1 */
	/* USER CODE BEGIN 2 */
	init();		// Init HAL Arduino
	BSP_LED_Init(LED2);

    Wire.begin();
    writePort8574( SLAVE_ADDR_8574_B , 0xff); //configure le composant B en entrée

	Serial.begin(9600);
	Serial.println("tests filtrage + i2c");
	
	analogWriteResolution(resConv);
	analogReadResolution(resConv);

	/* USER CODE END 2 */

	/* Infinite loop */
	/* USER CODE BEGIN WHILE */

	/* Infinite loop */
	while (1)
	{
		static uint32_t now,TechPred=0;	// pour la gestion de l'instant déchantillonnage
		float ek, sk;			// échantillons de l'équation de récurrence
		static float ek_1 = 0, ek_2 = 0;			

		now = micros();
		if ( (uint32_t)(now-TechPred) > Tech )		// une Tech s'est écoulée
		{
			TechPred = now ;
			BSP_LED_Toggle(LED2);

			// Conversion A -> N
			ek = (int32_t)analogRead(ADCPin) - offsetNum;	// Attention, sur Arduino, la résolution du CAN est par défaut 10 bits

			// Calcul de l'équation
			sk = 0.5*ek + 0.5*ek_1 ;

			// Mise à jour des échantillons mémorisés
			ek_1 = ek ;

			// Conversion N -> A
			analogWrite(DACPin, sk + offsetNum) ;	// Attention, sur Arduino, la résolution du CNA est par défaut sur 8 bits : analogWrite(DACPin, sk/4)


		}
		else		// Ajout rapide du test i2c
		{
			char  val;
			if (readPort8574(SLAVE_ADDR_8574_B,&val)==0)
			{
				Serial.print("lecture  8574  OK, ");
				if (writePort8574(SLAVE_ADDR_8574_A,val)==0)
					Serial.println("ecriture 8574  OK");
				else
					Serial.println("ecriture 8574 NOK");
			}
			else
				Serial.println("lecture  8574 NOK,");
			/*
		    writePort8574(SLAVE_ADDR_8574_A, 0x55);
		    delay(500);
		    writePort8574(SLAVE_ADDR_8574_A, 0xAA);
		    delay(500);
			 */
		}
//		{
//			// gestion i2C
//			   {
//			      byte error, address;
//			      int nDevices;
//
//			      Serial.println("Scanning...");
//
//			      nDevices = 0;
//			      for(address = 1; address < 127; address++ )
//			      {
//			        // The i2c_scanner uses the return value of
//			        // the Write.endTransmisstion to see if
//			        // a device did acknowledge to the address.
//			        Wire.beginTransmission(address);
//			        error = Wire.endTransmission();
//
//			        if (error == 0)
//			        {
//			          Serial.print("I2C device found at address 0x");
//			          if (address<16)
//			            Serial.print("0");
//			          Serial.print(address,HEX);
//			          Serial.println("  !");
//
//			          nDevices++;
//			        }
//			        else if (error==4)
//			        {
//			          Serial.print("Unknow error at address 0x");
//			          if (address<16)
//			            Serial.print("0");
//			          Serial.println(address,HEX);
//			        }
//			      }
//			      if (nDevices == 0)
//			        Serial.println("No I2C devices found\n");
//			      else
//			        Serial.println("done\n");
//
//			      delay(5000);           // wait 5 seconds for next scan
//			    }
//		}

		/* USER CODE END WHILE */

		/* USER CODE BEGIN 3 */
	}
	/* USER CODE END 3 */
}

/**
 * @brief System Clock Configuration
 * @retval None
 */
void SystemClock_Config(void)
{
	RCC_OscInitTypeDef RCC_OscInitStruct = {0};
	RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
	RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

	/** Initializes the CPU, AHB and APB busses clocks
	 */
	RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
	RCC_OscInitStruct.HSIState = RCC_HSI_ON;
	RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
	RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
	RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
	RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL16;
	if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
	{
		Error_Handler();
	}
	/** Initializes the CPU, AHB and APB busses clocks
	 */
	RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
			|RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
	RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
	RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
	RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
	RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

	if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
	{
		Error_Handler();
	}
	PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_ADC12;
	PeriphClkInit.Adc12ClockSelection = RCC_ADC12PLLCLK_DIV1;
	if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
	{
		Error_Handler();
	}
}


/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
 * @brief  This function is executed in case of error occurrence.
 * @retval None
 */

#ifdef  USE_FULL_ASSERT
/**
 * @brief  Reports the name of the source file and the source line number
 *         where the assert_param error has occurred.
 * @param  file: pointer to the source file name
 * @param  line: assert_param error line source number
 * @retval None
 */
void assert_failed(char *file, uint32_t line)
{ 
	/* USER CODE BEGIN 6 */
	/* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
	/* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
